@@ -16,11 +16,11 @@ class GreaderFeed : public Feed {
     explicit GreaderFeed(RootItem* parent = nullptr);
 
     virtual bool canBeDeleted() const;
-    virtual bool deleteItem();
+    virtual void deleteItem();
 
   private:
     GreaderServiceRoot* serviceRoot() const;
-    bool removeItself();
+    void removeItself();
 };
 
 #endif // TTRSSFEED_H
