@@ -5,7 +5,6 @@
 #include "src/definitions.h"
 #include "src/greaderfeed.h"
 
-#include <librssguard/3rd-party/boolinq/boolinq.h>
 #include <librssguard/database/databasequeries.h>
 #include <librssguard/exceptions/applicationexception.h>
 #include <librssguard/exceptions/feedfetchexception.h>
@@ -16,6 +15,7 @@
 #include <librssguard/network-web/oauth2service.h>
 #include <librssguard/network-web/webfactory.h>
 #include <librssguard/services/abstract/labelsnode.h>
+#include <qtlinq/qtlinq.h>
 
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -309,13 +309,13 @@ QList<Message> GreaderNetwork::getMessagesIntelligently(ServiceRoot* root,
   }
 
   // Add prefetched messages.
-  auto iter = boolinq::from(msgs);
+  auto iter = qlinq::from(msgs);
   QMutexLocker mtx(&m_mutexPrefetchedMessages);
 
   for (int i = 0; i < m_prefetchedMessages.size(); i++) {
     auto prefetched_msg = m_prefetchedMessages.at(i);
 
-    if (prefetched_msg.m_feedId == stream_id && !iter.any([&prefetched_msg](const Message& ms) {
+    if (prefetched_msg.m_feedCustomId == stream_id && !iter.any([&prefetched_msg](const Message& ms) {
           return ms.m_customId == prefetched_msg.m_customId;
         })) {
       msgs.append(prefetched_msg);
@@ -542,20 +542,19 @@ QList<Message> GreaderNetwork::itemContents(ServiceRoot* root,
         full_url += QSL("&c=%1").arg(continuation);
       }
 
-      std::list inp = boolinq::from(batch_ids)
-                        .select([this](const QString& id) {
-                          return QSL("i=%1").arg(m_service == GreaderServiceRoot::Service::TheOldReader
-                                                   ? id
-                                                   : QUrl::toPercentEncoding(id));
-                        })
-                        .toStdList();
-      QStringList inp_s = FROM_STD_LIST(QStringList, inp);
+      auto inp = qlinq::from(batch_ids)
+                   .select([this](const QString& id) {
+                     return QSL("i=%1").arg(m_service == GreaderServiceRoot::Service::TheOldReader
+                                              ? id
+                                              : QUrl::toPercentEncoding(id));
+                   })
+                   .toList();
 
       if (m_service == GreaderServiceRoot::Service::Reedah || m_service == GreaderServiceRoot::Service::Miniflux) {
-        inp_s.append(tokenParameter());
+        inp.append(tokenParameter());
       }
 
-      QByteArray input = inp_s.join(QSL("&")).toUtf8();
+      QByteArray input = inp.join(QSL("&")).toUtf8();
       QByteArray output_stream;
       auto result_stream =
         NetworkFactory::performNetworkOperation(full_url,
@@ -745,7 +744,6 @@ RootItem* GreaderNetwork::decodeTagsSubscriptions(const QString& categories,
 
     if ((label[QSL("type")].toString() == QSL("folder") ||
          (m_service == GreaderServiceRoot::Service::TheOldReader && label_id.contains(QSL("/label/"))))) {
-
       // We have category (not "state" or "tag" or "label").
       auto* category = new Category();
 
@@ -758,7 +756,7 @@ RootItem* GreaderNetwork::decodeTagsSubscriptions(const QString& categories,
     }
     else if (label[QSL("type")] == QSL("tag")) {
       QString plain_name = QRegularExpression(".+\\/([^\\/]+)").match(label_id).captured(1);
-      auto* new_lbl = new Label(plain_name, TextFactory::generateColorFromText(label_id));
+      auto* new_lbl = new Label(plain_name, IconFactory::fromColor(TextFactory::generateColorFromText(label_id)));
 
       new_lbl->setCustomId(label_id);
       lbls.append(new_lbl);
@@ -769,7 +767,7 @@ RootItem* GreaderNetwork::decodeTagsSubscriptions(const QString& categories,
       if (!cats.contains(label_id)) {
         // This stream is not a category, it is label, bitches!
         QString plain_name = QRegularExpression(QSL(".+\\/([^\\/]+)")).match(label_id).captured(1);
-        auto* new_lbl = new Label(plain_name, TextFactory::generateColorFromText(label_id));
+        auto* new_lbl = new Label(plain_name, IconFactory::fromColor(TextFactory::generateColorFromText(label_id)));
 
         new_lbl->setCustomId(label_id);
         lbls.append(new_lbl);
@@ -883,7 +881,7 @@ QNetworkReply::NetworkError GreaderNetwork::clientLogin(const QNetworkProxy& pro
     // Save credentials.
     auto lines = QString::fromUtf8(output).replace(QSL("\r"), QString()).split('\n');
 
-    for (const QString& line : lines) {
+    for (const QString& line : std::as_const(lines)) {
       int eq = line.indexOf('=');
 
       if (eq > 0) {
@@ -1029,12 +1027,6 @@ QString GreaderNetwork::convertShortStreamIdToLongStreamId(const QString& stream
   }
 }
 
-QString GreaderNetwork::simplifyStreamId(const QString& stream_id) const {
-  static QRegularExpression reg(QSL("\\/\\d+\\/"));
-
-  return QString(stream_id).replace(reg, QSL("/-/"));
-}
-
 QStringList GreaderNetwork::decodeItemIds(const QString& stream_json_data, QString& continuation) {
   QStringList ids;
   QJsonDocument json_doc = QJsonDocument::fromJson(stream_json_data.toUtf8());
@@ -1043,7 +1035,7 @@ QStringList GreaderNetwork::decodeItemIds(const QString& stream_json_data, QStri
   continuation = json_doc.object()[QSL("continuation")].toString();
   ids.reserve(json.count());
 
-  for (const QJsonValue& id : json) {
+  for (const QJsonValue& id : std::as_const(json)) {
     ids.append(id.toObject()[QSL("id")].toString());
   }
 
@@ -1062,13 +1054,19 @@ QList<Message> GreaderNetwork::decodeStreamContents(ServiceRoot* root,
   continuation = json_doc.object()[QSL("continuation")].toString();
   messages.reserve(json.count());
 
-  for (const QJsonValue& obj : json) {
+  for (const QJsonValue& obj : std::as_const(json)) {
     auto message_obj = obj.toObject();
     Message message;
 
     message.m_title = message_obj[QSL("title")].toString();
     message.m_author = message_obj[QSL("author")].toString();
-    message.m_created = QDateTime::fromSecsSinceEpoch(message_obj[QSL("published")].toInt(), Qt::TimeSpec::UTC);
+    message.m_created = QDateTime::fromSecsSinceEpoch(message_obj[QSL("published")].toInt(),
+#if QT_VERSION >= 0x060700 // Qt >= 6.7.0
+                                                      QTimeZone::utc());
+#else
+                                                      Qt::TimeSpec::UTC);
+#endif
+
     message.m_createdFromFeed = true;
     message.m_customId = message_obj[QSL("id")].toString();
 
@@ -1076,7 +1074,7 @@ QList<Message> GreaderNetwork::decodeStreamContents(ServiceRoot* root,
     auto enclosures = message_obj[QSL("enclosure")].toArray();
     auto categories = message_obj[QSL("categories")].toArray();
 
-    for (const QJsonValue& alt : alternates) {
+    for (const QJsonValue& alt : std::as_const(alternates)) {
       auto alt_obj = alt.toObject();
       QString mime = alt_obj[QSL("type")].toString();
       QString href = alt_obj[QSL("href")].toString();
@@ -1085,19 +1083,19 @@ QList<Message> GreaderNetwork::decodeStreamContents(ServiceRoot* root,
         message.m_url = href;
       }
       else {
-        message.m_enclosures.append(Enclosure(href, mime));
+        message.m_enclosures.append(QSharedPointer<MessageEnclosure>(new MessageEnclosure(href, mime)));
       }
     }
 
-    for (const QJsonValue& enc : enclosures) {
+    for (const QJsonValue& enc : std::as_const(enclosures)) {
       auto enc_obj = enc.toObject();
       QString mime = enc_obj[QSL("type")].toString();
       QString href = enc_obj[QSL("href")].toString();
 
-      message.m_enclosures.append(Enclosure(href, mime));
+      message.m_enclosures.append(QSharedPointer<MessageEnclosure>(new MessageEnclosure(href, mime)));
     }
 
-    for (const QJsonValue& cat : categories) {
+    for (const QJsonValue& cat : std::as_const(categories)) {
       QString category = cat.toString();
 
       if (category.endsWith(QSL(GREADER_API_STATE_READ))) {
@@ -1107,20 +1105,20 @@ QList<Message> GreaderNetwork::decodeStreamContents(ServiceRoot* root,
         message.m_isImportant = true;
       }
       else if (category.contains(QSL("label"))) {
-        Label* label = boolinq::from(active_labels.begin(), active_labels.end()).firstOrDefault([category](Label* lbl) {
+        auto label = qlinq::from(active_labels).firstOrDefault([category](Label* lbl) {
           return lbl->customId() == category;
         });
 
-        if (label != nullptr) {
+        if (label.has_value()) {
           // We found live Label object for our assigned label.
-          message.m_assignedLabels.append(label);
+          message.m_assignedLabels.append(label.value());
         }
       }
     }
 
     message.m_contents = message_obj[QSL("summary")].toObject()[QSL("content")].toString();
     message.m_rawContents = QJsonDocument(message_obj).toJson(QJsonDocument::JsonFormat::Compact);
-    message.m_feedId =
+    message.m_feedCustomId =
       stream_id.isEmpty() ? message_obj[QSL("origin")].toObject()[QSL("streamId")].toString() : stream_id;
 
     if (message.m_title.isEmpty()) {
@@ -1255,9 +1253,9 @@ void GreaderNetwork::initializeOauth() {
             Q_UNUSED(access_token)
 
             if (m_root != nullptr && m_root->accountId() > 0 && !refresh_token.isEmpty()) {
-              QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
-
-              DatabaseQueries::storeNewOauthTokens(database, refresh_token, m_root->accountId());
+              qApp->database()->worker()->write([&](const QSqlDatabase& db) {
+                DatabaseQueries::storeNewOauthTokens(db, refresh_token, m_root->accountId());
+              });
             }
           });
 }
