@@ -603,7 +603,7 @@ QList<Message> NextcloudGetMessagesResponse::messages() const {
     msg.m_created = TextFactory::parseDateTime(message_map[QSL("pubDate")].toDouble() * 1000);
     msg.m_createdFromFeed = true;
     msg.m_customId = message_map[QSL("id")].toVariant().toString();
-    msg.m_customHash = message_map[QSL("guidHash")].toString();
+    msg.m_customData = message_map[QSL("guidHash")].toString();
     msg.m_rawContents = QJsonDocument(message_map).toJson(QJsonDocument::JsonFormat::Compact);
 
     // In case body is empty, check for content in mediaDescription if item is available.
@@ -613,33 +613,30 @@ QList<Message> NextcloudGetMessagesResponse::messages() const {
 
     // Check for mediaThumbnail and append as first enclosure to be viewed in internal viewer.
     if (!message_map[QSL("mediaThumbnail")].isUndefined()) {
-      Enclosure enclosure;
+      MessageEnclosure* enclosure =
+        new MessageEnclosure(message_map[QSL("mediaThumbnail")].toString(), QSL("image/jpg"));
 
-      enclosure.m_mimeType = QSL("image/jpg");
-      enclosure.m_url = message_map[QSL("mediaThumbnail")].toString();
-
-      msg.m_enclosures.append(enclosure);
+      msg.m_enclosures.append(QSharedPointer<MessageEnclosure>(enclosure));
     }
 
     QString enclosure_link = message_map[QSL("enclosureLink")].toString();
 
     if (!enclosure_link.isEmpty()) {
-      Enclosure enclosure;
+      MessageEnclosure* enclosure = new MessageEnclosure();
 
-      enclosure.m_mimeType = message_map[QSL("enclosureMime")].toString();
-      enclosure.m_url = enclosure_link;
+      enclosure->setMimeType(message_map[QSL("enclosureMime")].toString());
+      enclosure->setUrl(enclosure_link);
 
-      if (enclosure.m_mimeType.isEmpty()) {
-        enclosure.m_mimeType = QSL("image/png");
+      if (enclosure->mimeType().isEmpty()) {
+        enclosure->setMimeType(QSL("image/png"));
       }
 
       if (!message_map[QSL("enclosureMime")].toString().isEmpty() ||
           !enclosure_link.startsWith(QSL("https://www.youtube.com/v/"))) {
-        msg.m_enclosures.append(enclosure);
+        msg.m_enclosures.append(QSharedPointer<MessageEnclosure>(enclosure));
       }
     }
 
-    msg.m_feedId = message_map[QSL("feedId")].toVariant().toString();
     msg.m_isImportant = message_map[QSL("starred")].toBool();
     msg.m_isRead = !message_map[QSL("unread")].toBool();
     msg.m_title = message_map[QSL("title")].toString();
