@@ -3,8 +3,6 @@
 #ifndef STANDARDFEEDEXPDETAILS_H
 #define STANDARDFEEDEXPDETAILS_H
 
-#include "src/standardfeed.h"
-
 #include "ui_standardfeedexpdetails.h"
 
 #include <QNetworkProxy>
@@ -21,9 +19,6 @@ class StandardFeedExpDetails : public QWidget {
   public:
     explicit StandardFeedExpDetails(QWidget* parent = nullptr);
 
-    void setHttp2Status(NetworkFactory::Http2Status status);
-    NetworkFactory::Http2Status http2Status() const;
-
   private:
     Ui::StandardFeedExpDetails m_ui;
 };
