@@ -10,8 +10,7 @@
 #include <librssguard/miscellaneous/application.h>
 #include <librssguard/miscellaneous/settings.h>
 #include <librssguard/miscellaneous/textfactory.h>
-
-#include <QTextCodec>
+#include <librssguard/miscellaneous/xmlencodingdetector.h>
 
 AtomParser::AtomParser(const QString& data) : FeedParser(data) {
   QString version = m_xml.documentElement().attribute(QSL("version"));
@@ -297,38 +296,19 @@ QList<StandardFeed*> AtomParser::discoverFeeds(ServiceRoot* root, const QUrl& ur
 
 QPair<StandardFeed*, QList<IconLocation>> AtomParser::guessFeed(const QByteArray& content,
                                                                 const NetworkResult& network_res) const {
-  QString xml_schema_encoding = QSL(DEFAULT_FEED_ENCODING);
-  QString xml_contents_encoded;
-  QString enc =
-    QRegularExpression(QSL("encoding=\"([A-Z0-9\\-]+)\""), QRegularExpression::PatternOption::CaseInsensitiveOption)
-      .match(content)
-      .captured(1);
-
-  if (!enc.isEmpty()) {
-    // Some "encoding" attribute was found get the encoding
-    // out of it.
-    xml_schema_encoding = enc;
-  }
-
-  QTextCodec* custom_codec = QTextCodec::codecForName(xml_schema_encoding.toLocal8Bit());
-
-  if (custom_codec != nullptr) {
-    xml_contents_encoded = custom_codec->toUnicode(content);
-  }
-  else {
-    xml_contents_encoded = QString::fromUtf8(content);
-  }
-
-  // NOTE: Some XMLs have whitespace before XML declaration, erase it.
-  xml_contents_encoded = xml_contents_encoded.trimmed();
+  QString encoding = XmlEncodingDetector::detectXmlEncoding(content);
+  QString xml_contents_encoded = TextFactory::fromEncoding(content, encoding);
 
   // Feed XML was obtained, guess it now.
-  QDomDocument xml_document;
+  DomDocument xml_document;
   QString error_msg;
   int error_line, error_column;
 
   if (!xml_document.setContent(xml_contents_encoded, true, &error_msg, &error_line, &error_column)) {
-    throw ApplicationException(QObject::tr("XML is not well-formed, %1").arg(error_msg));
+    throw ApplicationException(QObject::tr("XML is not well-formed, %1, line %2, column %3")
+                                 .arg(error_msg)
+                                 .arg(QString::number(error_line))
+                                 .arg(QString::number(error_column)));
   }
 
   QDomElement root_element = xml_document.documentElement();
@@ -340,7 +320,7 @@ QPair<StandardFeed*, QList<IconLocation>> AtomParser::guessFeed(const QByteArray
   auto* feed = new StandardFeed();
   QList<IconLocation> icon_possible_locations;
 
-  feed->setEncoding(xml_schema_encoding);
+  feed->setEncoding(encoding);
   feed->setType(StandardFeed::Type::Atom10);
   feed->setTitle(root_element.namedItem(QSL("title")).toElement().text());
   feed->setDescription(root_element.namedItem(QSL("subtitle")).toElement().text());
@@ -394,6 +374,49 @@ QString AtomParser::atomNamespace() const {
   return m_atomNamespace;
 }
 
+QList<FeedComment> AtomParser::comments(const QDomElement& msg_element) const {
+  QDomNodeList links = msg_element.elementsByTagNameNS(m_atomNamespace, QSL("link"));
+
+  for (int i = 0; i < links.size(); i++) {
+    QDomElement link_elem = links.at(i).toElement();
+
+    if (link_elem.attribute(QSL("rel")) != QSL("replies") ||
+        link_elem.attribute(QSL("type")) != QSL("application/atom+xml")) {
+      continue;
+    }
+
+    QString comments_atom = link_elem.attribute(QSL("href"));
+
+    if (comments_atom.isEmpty()) {
+      continue;
+    }
+
+    QByteArray comments_atom_data = m_resourceHandler(comments_atom);
+    AtomParser atom_parser(QString::fromUtf8(comments_atom_data));
+    QList<Message> extracted_comments = atom_parser.messages();
+
+    if (extracted_comments.isEmpty()) {
+      return {};
+    }
+
+    QList<FeedComment> cmnts;
+    cmnts.reserve(extracted_comments.size());
+
+    for (Message& extracted_comment : extracted_comments) {
+      extracted_comment.sanitize(nullptr, false);
+
+      FeedComment cmnt;
+      cmnt.m_title = extracted_comment.m_title;
+      cmnt.m_contents = extracted_comment.m_contents;
+      cmnts.append(cmnt);
+    }
+
+    return cmnts;
+  }
+
+  return {};
+}
+
 QDomNodeList AtomParser::xmlMessageElements() {
   return m_xml.elementsByTagNameNS(m_atomNamespace, QSL("entry"));
 }
@@ -413,17 +436,22 @@ QString AtomParser::xmlMessageDescription(const QDomElement& msg_element) const
     }
   }
 
+  if (fetchComments()) {
+    summary += formatComments(comments(msg_element));
+  }
+
   return summary;
 }
 
 QDateTime AtomParser::xmlMessageDateCreated(const QDomElement& msg_element) {
+  QString published = xmlTextsFromPath(msg_element, m_atomNamespace, QSL("published"), true).join(QSL(", "));
   QString updated = xmlTextsFromPath(msg_element, m_atomNamespace, QSL("updated"), true).join(QSL(", "));
 
-  if (updated.simplified().isEmpty()) {
+  if (updated.trimmed().isEmpty()) {
     updated = xmlTextsFromPath(msg_element, m_atomNamespace, QSL("modified"), true).join(QSL(", "));
   }
 
-  return TextFactory::parseDateTime(updated, &m_dateTimeFormat);
+  return decideArticleDate(published, updated);
 }
 
 QString AtomParser::xmlMessageId(const QDomElement& msg_element) const {
@@ -454,8 +482,8 @@ QString AtomParser::xmlMessageUrl(const QDomElement& msg_element) const {
   }
 }
 
-QList<Enclosure> AtomParser::xmlMessageEnclosures(const QDomElement& msg_element) const {
-  QList<Enclosure> enclosures;
+QList<QSharedPointer<MessageEnclosure>> AtomParser::xmlMessageEnclosures(const QDomElement& msg_element) const {
+  QList<QSharedPointer<MessageEnclosure>> enclosures;
   QDomNodeList elem_links = msg_element.elementsByTagNameNS(m_atomNamespace, QSL("link"));
 
   for (int i = 0; i < elem_links.size(); i++) {
@@ -463,15 +491,16 @@ QList<Enclosure> AtomParser::xmlMessageEnclosures(const QDomElement& msg_element
     QString attribute = link.attribute(QSL("rel"));
 
     if (attribute == QSL("enclosure")) {
-      enclosures.append(Enclosure(link.attribute(QSL("href")), link.attribute(QSL("type"))));
+      enclosures.append(QSharedPointer<MessageEnclosure>(new MessageEnclosure(link.attribute(QSL("href")),
+                                                                              link.attribute(QSL("type")))));
     }
   }
 
   return enclosures;
 }
 
-QList<MessageCategory> AtomParser::xmlMessageCategories(const QDomElement& msg_element) const {
-  QList<MessageCategory> cats;
+QList<QSharedPointer<MessageCategory>> AtomParser::xmlMessageCategories(const QDomElement& msg_element) const {
+  QList<QSharedPointer<MessageCategory>> cats;
   QDomNodeList elem_cats = msg_element.toElement().elementsByTagNameNS(m_atomNamespace, QSL("category"));
 
   for (int i = 0; i < elem_cats.size(); i++) {
@@ -479,7 +508,7 @@ QList<MessageCategory> AtomParser::xmlMessageCategories(const QDomElement& msg_e
     QString lbl = cat.attribute(QSL("label"));
     QString term = cat.attribute(QSL("term"));
 
-    cats.append(MessageCategory(lbl.isEmpty() ? term : lbl));
+    cats.append(QSharedPointer<MessageCategory>(new MessageCategory(lbl.isEmpty() ? term : lbl)));
   }
 
   return cats;
