@@ -7,7 +7,6 @@
 
 #include <librssguard/core/message.h>
 
-#include <QDomDocument>
 #include <QList>
 
 class AtomParser : public FeedParser {
@@ -29,12 +28,13 @@ class AtomParser : public FeedParser {
     virtual QDateTime xmlMessageDateCreated(const QDomElement& msg_element);
     virtual QString xmlMessageId(const QDomElement& msg_element) const;
     virtual QString xmlMessageUrl(const QDomElement& msg_element) const;
-    virtual QList<Enclosure> xmlMessageEnclosures(const QDomElement& msg_element) const;
-    virtual QList<MessageCategory> xmlMessageCategories(const QDomElement& msg_element) const;
+    virtual QList<QSharedPointer<MessageEnclosure>> xmlMessageEnclosures(const QDomElement& msg_element) const;
+    virtual QList<QSharedPointer<MessageCategory>> xmlMessageCategories(const QDomElement& msg_element) const;
     virtual QString xmlMessageAuthor(const QDomElement& msg_element) const;
 
   private:
     QString atomNamespace() const;
+    QList<FeedComment> comments(const QDomElement& msg_element) const;
 
     QString m_atomNamespace;
 };
