@@ -27,7 +27,13 @@ class RssParser : public FeedParser {
     virtual QDateTime xmlMessageDateCreated(const QDomElement& msg_element);
     virtual QString xmlMessageId(const QDomElement& msg_element) const;
     virtual QString xmlMessageUrl(const QDomElement& msg_element) const;
-    virtual QList<Enclosure> xmlMessageEnclosures(const QDomElement& msg_element) const;
+    virtual QList<QSharedPointer<MessageEnclosure>> xmlMessageEnclosures(const QDomElement& msg_element) const;
+    virtual QList<QSharedPointer<MessageCategory>> xmlMessageCategories(const QDomElement& msg_element) const;
+
+  private:
+    QList<FeedComment> comments(const QDomElement& msg_element) const;
+
+    QString m_wfwNamespace;
 };
 
 #endif // RSSPARSER_H
