@@ -7,14 +7,14 @@
 #include "src/standardfeed.h"
 #include "src/standardserviceroot.h"
 
-#include <librssguard/3rd-party/boolinq/boolinq.h>
 #include <librssguard/definitions/definitions.h>
 #include <librssguard/exceptions/applicationexception.h>
 #include <librssguard/miscellaneous/application.h>
+#include <librssguard/miscellaneous/domdocument.h>
 #include <librssguard/miscellaneous/iconfactory.h>
+#include <qtlinq/qtlinq.h>
 
 #include <QDomAttr>
-#include <QDomDocument>
 #include <QDomElement>
 #include <QLocale>
 #include <QSqlDatabase>
@@ -24,14 +24,13 @@
 
 FeedsImportExportModel::FeedsImportExportModel(StandardServiceRoot* account, QObject* parent)
   : AccountCheckSortedModel(parent), m_account(account), m_mode(Mode::Import), m_newRoot(nullptr) {
-
   connect(&m_watcherLookup, &QFutureWatcher<bool>::progressValueChanged, this, [=](int prog) {
     emit parsingProgress(prog, m_lookup.size());
   });
 
   connect(&m_watcherLookup, &QFutureWatcher<bool>::finished, this, [=]() {
     auto res = m_watcherLookup.future().results();
-    int number_error = boolinq::from(res).count(false);
+    int number_error = qlinq::from(res).count(false);
 
     emit layoutAboutToBeChanged();
     setRootItem(m_newRoot);
@@ -61,7 +60,7 @@ FeedsImportExportModel::~FeedsImportExportModel() {
 }
 
 bool FeedsImportExportModel::exportToOMPL20(QByteArray& result, bool export_icons) {
-  QDomDocument opml_document;
+  DomDocument opml_document;
   QDomProcessingInstruction xml_declaration =
     opml_document.createProcessingInstruction(QSL("xml"), QSL("version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\""));
 
@@ -70,7 +69,11 @@ bool FeedsImportExportModel::exportToOMPL20(QByteArray& result, bool export_icon
   // Added OPML 2.0 metadata.
   opml_document.appendChild(opml_document.createElement(QSL("opml")));
   opml_document.documentElement().setAttribute(QSL("version"), QSL("2.0"));
+
+#if QT_VERSION_MAJOR == 5
   opml_document.documentElement().setAttribute(QSL("xmlns:rssguard"), QSL(APP_URL));
+#endif
+
   QDomElement elem_opml_head = opml_document.createElement(QSL("head"));
   QDomElement elem_opml_title = opml_document.createElement(QSL("title"));
   QDomText text_opml_title = opml_document.createTextNode(QSL(APP_NAME));
@@ -112,8 +115,9 @@ bool FeedsImportExportModel::exportToOMPL20(QByteArray& result, bool export_icon
           outline_category.setAttribute(QSL("description"), child_item->description());
 
           if (export_icons && !child_item->icon().isNull()) {
-            outline_category.setAttribute(QSL("rssguard:icon"),
-                                          QString(qApp->icons()->toByteArray(child_item->icon())));
+            outline_category.setAttributeNS(QSL(APP_URL),
+                                            QSL("rssguard:icon"),
+                                            QString(qApp->icons()->toByteArray(child_item->icon())));
           }
 
           active_element.appendChild(outline_category);
@@ -133,11 +137,15 @@ bool FeedsImportExportModel::exportToOMPL20(QByteArray& result, bool export_icon
           outline_feed.setAttribute(QSL("encoding"), child_feed->encoding());
           outline_feed.setAttribute(QSL("title"), child_feed->title());
 
-          outline_feed.setAttribute(QSL("rssguard:xmlUrlType"), QString::number(int(child_feed->sourceType())));
-          outline_feed.setAttribute(QSL("rssguard:postProcess"), child_feed->postProcessScript());
+          outline_feed.setAttributeNS(QSL(APP_URL),
+                                      QSL("rssguard:xmlUrlType"),
+                                      QString::number(int(child_feed->sourceType())));
+          outline_feed.setAttributeNS(QSL(APP_URL), QSL("rssguard:postProcess"), child_feed->postProcessScript());
 
           if (export_icons && !child_feed->icon().isNull()) {
-            outline_feed.setAttribute(QSL("rssguard:icon"), QString(qApp->icons()->toByteArray(child_feed->icon())));
+            outline_feed.setAttributeNS(QSL(APP_URL),
+                                        QSL("rssguard:icon"),
+                                        QString(qApp->icons()->toByteArray(child_feed->icon())));
           }
 
           switch (child_feed->type()) {
@@ -235,6 +243,8 @@ bool FeedsImportExportModel::produceFeed(const FeedLookup& feed_lookup) {
         if (feed_lookup.add_errored_feeds) {
           // Feed guessing failed, add like regular feed anyway.
           new_feed = new StandardFeed();
+          new_feed->setStatus(Feed::Status::OtherError);
+
           fillFeedFromFeedLookupData(new_feed, feed_lookup);
         }
         else {
@@ -249,7 +259,6 @@ bool FeedsImportExportModel::produceFeed(const FeedLookup& feed_lookup) {
 
     QMutexLocker mtx(&m_mtxLookup);
     feed_lookup.parent->appendChild(new_feed);
-
     return true;
   }
   catch (const ApplicationException& ex) {
@@ -321,9 +330,9 @@ void FeedsImportExportModel::importAsOPML20(const QByteArray& data,
   setRootItem(nullptr);
   emit layoutChanged();
 
-  QDomDocument opml_document;
+  DomDocument opml_document;
 
-  if (!opml_document.setContent(data)) {
+  if (!opml_document.setContent(data, true)) {
     throw ApplicationException(tr("OPML document contains errors"));
   }
 
@@ -382,11 +391,11 @@ void FeedsImportExportModel::importAsOPML20(const QByteArray& data,
             feed_data[QSL("type")] = child_element.attribute(QSL("version"), QSL(DEFAULT_FEED_TYPE)).toUpper();
             feed_data[QSL("description")] = child_element.attribute(QSL("description"));
             feed_data[QSL("icon")] =
-              qApp->icons()->fromByteArray(child_element.attribute(QSL("rssguard:icon")).toLocal8Bit());
+              qApp->icons()->fromByteArray(child_element.attributeNS(QSL(APP_URL), QSL("icon")).toLocal8Bit());
             feed_data[QSL("sourceType")] =
-              QVariant::fromValue(StandardFeed::SourceType(child_element.attribute(QSL("rssguard:xmlUrlType"))
+              QVariant::fromValue(StandardFeed::SourceType(child_element.attributeNS(QSL(APP_URL), QSL("xmlUrlType"))
                                                              .toInt()));
-            feed_data[QSL("postProcessScript")] = child_element.attribute(QSL("rssguard:postProcess"));
+            feed_data[QSL("postProcessScript")] = child_element.attributeNS(QSL(APP_URL), QSL("postProcess"));
 
             f.custom_proxy = custom_proxy;
             f.fetch_metadata_online = fetch_metadata_online;
@@ -406,7 +415,7 @@ void FeedsImportExportModel::importAsOPML20(const QByteArray& data,
           QString category_title = child_element.attribute(QSL("text"));
           QString category_description = child_element.attribute(QSL("description"));
           QIcon category_icon =
-            qApp->icons()->fromByteArray(child_element.attribute(QSL("rssguard:icon")).toLocal8Bit());
+            qApp->icons()->fromByteArray(child_element.attributeNS(QSL(APP_URL), QSL("icon")).toLocal8Bit());
 
           if (category_title.isEmpty()) {
             qWarningNN << LOGSEC_STANDARD
@@ -414,7 +423,7 @@ void FeedsImportExportModel::importAsOPML20(const QByteArray& data,
             category_title = child_element.attribute(QSL("title"));
 
             if (category_title.isEmpty()) {
-              category_title = tr("Category ") + QString::number(QDateTime::currentDateTime().toMSecsSinceEpoch());
+              category_title = tr("Folder ") + QString::number(QDateTime::currentDateTime().toMSecsSinceEpoch());
             }
           }
 
