@@ -15,8 +15,13 @@ class FormLog : public QDialog {
     virtual ~FormLog();
 
   public slots:
+    void clearLog();
     void appendLogMessage(const QString& message);
 
+  protected:
+    virtual void closeEvent(QCloseEvent* event);
+    virtual void keyPressEvent(QKeyEvent* event);
+
   private:
     Ui::FormLog m_ui;
 };
