@@ -2,11 +2,14 @@
 
 #include "gui/dialogs/formmessagefiltersmanager.h"
 
-#include "3rd-party/boolinq/boolinq.h"
-#include "core/messagefilter.h"
-#include "core/messagesforfiltersmodel.h"
+#include "core/messagesmodel.h"
 #include "database/databasequeries.h"
 #include "exceptions/filteringexception.h"
+#include "filtering/filteringsystem.h"
+#include "filtering/filterobjects.h"
+#include "filtering/messagefilter.h"
+#include "filtering/messagesforfiltersmodel.h"
+#include "gui/dialogs/filedialog.h"
 #include "gui/guiutilities.h"
 #include "gui/messagebox.h"
 #include "gui/reusable/jssyntaxhighlighter.h"
@@ -14,33 +17,41 @@
 #include "miscellaneous/feedreader.h"
 #include "miscellaneous/iconfactory.h"
 #include "network-web/webfactory.h"
+#include "qtlinq/qtlinq.h"
 #include "services/abstract/accountcheckmodel.h"
 #include "services/abstract/feed.h"
 
 #include <QDateTime>
 #include <QJSEngine>
 #include <QProcess>
+#include <QSortFilterProxyModel>
 
 FormMessageFiltersManager::FormMessageFiltersManager(FeedReader* reader,
                                                      const QList<ServiceRoot*>& accounts,
                                                      QWidget* parent)
   : QDialog(parent), m_feedsModel(new AccountCheckSortedModel(this)), m_rootItem(new RootItem()), m_accounts(accounts),
-    m_reader(reader), m_loadingFilter(false), m_msgModel(new MessagesForFiltersModel(this)) {
+    m_reader(reader), m_loadingFilter(false), m_msgProxyModel(new QSortFilterProxyModel(this)),
+    m_msgModel(new MessagesForFiltersModel(this)) {
   m_ui.setupUi(this);
 
+  m_defaultTextColor = m_ui.m_txtErrors->textColor();
   m_highlighter = new JsSyntaxHighlighter(m_ui.m_txtScript->document());
 
   std::sort(m_accounts.begin(), m_accounts.end(), [](const ServiceRoot* lhs, const ServiceRoot* rhs) {
     return lhs->title().compare(rhs->title(), Qt::CaseSensitivity::CaseInsensitive) < 0;
   });
 
-  // TODO: Add sorting.
-  m_ui.m_treeExistingMessages->setModel(m_msgModel);
+  m_msgProxyModel->setSortCaseSensitivity(Qt::CaseSensitivity::CaseInsensitive);
+  m_msgProxyModel->setSourceModel(m_msgModel);
+  m_ui.m_treeExistingMessages->setModel(m_msgProxyModel);
 
   GuiUtilities::applyDialogProperties(*this, qApp->icons()->fromTheme(QSL("view-list-details")));
 
   m_ui.m_treeFeeds->setIndentation(FEEDS_VIEW_INDENTATION);
   m_ui.m_treeFeeds->setModel(m_feedsModel);
+
+  m_ui.m_btnImport->setIcon(qApp->icons()->fromTheme(QSL("document-import")));
+  m_ui.m_btnExport->setIcon(qApp->icons()->fromTheme(QSL("document-export")));
   m_ui.m_btnCheckAll->setIcon(qApp->icons()->fromTheme(QSL("dialog-yes"), QSL("edit-select-all")));
   m_ui.m_btnUncheckAll->setIcon(qApp->icons()->fromTheme(QSL("dialog-no"), QSL("edit-select-none")));
   m_ui.m_btnAddNew->setIcon(qApp->icons()->fromTheme(QSL("list-add")));
@@ -49,32 +60,44 @@ FormMessageFiltersManager::FormMessageFiltersManager(FeedReader* reader,
   m_ui.m_btnTest->setIcon(qApp->icons()->fromTheme(QSL("media-playback-start")));
   m_ui.m_btnRunOnMessages->setIcon(qApp->icons()->fromTheme(QSL("media-playback-start")));
   m_ui.m_btnDetailedHelp->setIcon(qApp->icons()->fromTheme(QSL("help-contents")));
+
+  m_ui.m_btnUp->setIcon(qApp->icons()->fromTheme(QSL("arrow-up"), QSL("go-up")));
+  m_ui.m_btnDown->setIcon(qApp->icons()->fromTheme(QSL("arrow-down"), QSL("go-down")));
+  m_ui.m_btnEnable->setIcon(qApp->icons()->fromTheme(QSL("dialog-ok"), QSL("media-playback-start")));
+
   m_ui.m_txtScript->setFont(QFontDatabase::systemFont(QFontDatabase::SystemFont::FixedFont));
+  m_ui.m_tbMessageContents->setFont(QFontDatabase::systemFont(QFontDatabase::SystemFont::FixedFont));
+  m_ui.m_txtErrors->setFont(QFontDatabase::systemFont(QFontDatabase::SystemFont::FixedFont));
   m_ui.m_treeExistingMessages->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
 
+  m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_RESULT,
+                                                              QHeaderView::ResizeMode::ResizeToContents);
   m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_ISREAD,
                                                               QHeaderView::ResizeMode::ResizeToContents);
   m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_ISIMPORTANT,
                                                               QHeaderView::ResizeMode::ResizeToContents);
   m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_ISDELETED,
                                                               QHeaderView::ResizeMode::ResizeToContents);
-  m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_AUTHOR,
-                                                              QHeaderView::ResizeMode::ResizeToContents);
   m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_CREATED,
                                                               QHeaderView::ResizeMode::ResizeToContents);
   m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_SCORE,
                                                               QHeaderView::ResizeMode::ResizeToContents);
-  m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_TITLE, QHeaderView::ResizeMode::Interactive);
-  m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_URL, QHeaderView::ResizeMode::Interactive);
+  m_ui.m_treeExistingMessages->header()->setSectionResizeMode(MFM_MODEL_TITLE, QHeaderView::ResizeMode::Stretch);
+  m_ui.m_treeExistingMessages->header()->setSectionsMovable(false);
+  m_ui.m_treeExistingMessages->header()->setStretchLastSection(false);
 
-  connect(m_ui.m_btnDetailedHelp, &QPushButton::clicked, this, []() {
-    qApp->web()->openUrlInExternalBrowser(QSL(MSG_FILTERING_HELP));
-  });
+  connect(m_ui.m_btnExport, &QToolButton::clicked, this, &FormMessageFiltersManager::exportFilters);
+  connect(m_ui.m_btnImport, &QToolButton::clicked, this, &FormMessageFiltersManager::importFilters);
+
+  connect(m_ui.m_btnDown, &QToolButton::clicked, this, &FormMessageFiltersManager::moveFilterDown);
+  connect(m_ui.m_btnUp, &QToolButton::clicked, this, &FormMessageFiltersManager::moveFilterUp);
+  connect(m_ui.m_btnEnable, &QToolButton::clicked, this, &FormMessageFiltersManager::saveSelectedFilter);
+  connect(m_ui.m_btnDetailedHelp, &QPushButton::clicked, this, &FormMessageFiltersManager::openDocs);
   connect(m_ui.m_listFilters, &QListWidget::currentRowChanged, this, &FormMessageFiltersManager::loadFilter);
-  connect(m_ui.m_btnAddNew, &QPushButton::clicked, this, [this]() {
+  connect(m_ui.m_btnAddNew, &QToolButton::clicked, this, [this]() {
     addNewFilter();
   });
-  connect(m_ui.m_btnRemoveSelected, &QPushButton::clicked, this, &FormMessageFiltersManager::removeSelectedFilter);
+  connect(m_ui.m_btnRemoveSelected, &QToolButton::clicked, this, &FormMessageFiltersManager::removeSelectedFilter);
   connect(m_ui.m_txtTitle, &QLineEdit::textChanged, this, &FormMessageFiltersManager::saveSelectedFilter);
   connect(m_ui.m_txtScript, &QPlainTextEdit::textChanged, this, &FormMessageFiltersManager::saveSelectedFilter);
   connect(m_ui.m_btnTest, &QPushButton::clicked, this, &FormMessageFiltersManager::testFilter);
@@ -83,9 +106,9 @@ FormMessageFiltersManager::FormMessageFiltersManager(FeedReader* reader,
           static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
           this,
           &FormMessageFiltersManager::onAccountChanged);
-  connect(m_ui.m_btnCheckAll, &QPushButton::clicked, m_feedsModel->sourceModel(), &AccountCheckModel::checkAllItems);
+  connect(m_ui.m_btnCheckAll, &QToolButton::clicked, m_feedsModel->sourceModel(), &AccountCheckModel::checkAllItems);
   connect(m_ui.m_btnUncheckAll,
-          &QPushButton::clicked,
+          &QToolButton::clicked,
           m_feedsModel->sourceModel(),
           &AccountCheckModel::uncheckAllItems);
   connect(m_feedsModel->sourceModel(),
@@ -95,12 +118,16 @@ FormMessageFiltersManager::FormMessageFiltersManager(FeedReader* reader,
   connect(m_ui.m_treeFeeds->selectionModel(),
           &QItemSelectionModel::selectionChanged,
           this,
-          &FormMessageFiltersManager::displayMessagesOfFeed);
+          &FormMessageFiltersManager::onFeedChanged);
   connect(m_ui.m_btnRunOnMessages, &QPushButton::clicked, this, &FormMessageFiltersManager::processCheckedFeeds);
   connect(m_ui.m_treeExistingMessages,
           &QTreeView::customContextMenuRequested,
           this,
           &FormMessageFiltersManager::showMessageContextMenu);
+  connect(m_ui.m_treeExistingMessages->selectionModel(),
+          &QItemSelectionModel::currentRowChanged,
+          this,
+          &FormMessageFiltersManager::displaySelectedMessageDetails);
 
   connect(m_ui.m_searchWidget, &SearchTextWidget::searchCancelled, this, [this]() {
     m_ui.m_txtScript->find(QString());
@@ -113,8 +140,6 @@ FormMessageFiltersManager::FormMessageFiltersManager(FeedReader* reader,
   m_ui.m_txtScript->installEventFilter(this);
   m_ui.m_searchWidget->hide();
 
-  initializeTestingMessage();
-  initializePremadeFilters();
   loadFilters();
   loadFilter();
   loadAccounts();
@@ -157,6 +182,116 @@ bool FormMessageFiltersManager::eventFilter(QObject* watched, QEvent* event) {
   return false;
 }
 
+void FormMessageFiltersManager::importFilters() {
+  const QString filter = tr("Article filter files (*.json)");
+  const QString selected_file = FileDialog::openFileName(qApp->mainFormWidget(),
+                                                         tr("Select file article filters export"),
+                                                         qApp->documentsFolder(),
+                                                         QSL("filters.json"),
+                                                         filter,
+                                                         nullptr,
+                                                         GENERAL_REMEMBERED_PATH);
+
+  if (selected_file.isEmpty() || !QFile::exists(selected_file)) {
+    return;
+  }
+
+  try {
+    m_reader->importMessageFilters(IOFactory::readFile(selected_file));
+    loadFilters();
+  }
+  catch (const ApplicationException& ex) {
+    MsgBox::show(this,
+                 QMessageBox::Icon::Critical,
+                 tr("Error"),
+                 tr("Cannot export filters, error: '%1'.").arg(ex.message()));
+  }
+}
+
+void FormMessageFiltersManager::exportFilters() {
+  const QString filter = tr("Article filter files (*.json)");
+  const QString selected_file = FileDialog::saveFileName(qApp->mainFormWidget(),
+                                                         tr("Select file article filters export"),
+                                                         qApp->documentsFolder(),
+                                                         QSL("filters.json"),
+                                                         filter,
+                                                         nullptr,
+                                                         GENERAL_REMEMBERED_PATH);
+
+  if (selected_file.isEmpty()) {
+    return;
+  }
+
+  auto exported = m_reader->exportMessageFilters();
+
+  try {
+    IOFactory::writeFile(selected_file, exported);
+  }
+  catch (const ApplicationException& ex) {
+    MsgBox::show(this,
+                 QMessageBox::Icon::Critical,
+                 tr("Error"),
+                 tr("Cannot export filters, error: '%1'.").arg(ex.message()));
+  }
+}
+
+void FormMessageFiltersManager::moveFilterDown() {
+  auto* filter = selectedFilter();
+  auto row = m_ui.m_listFilters->currentRow();
+
+  qApp->database()->worker()->write([&](const QSqlDatabase& db) {
+    DatabaseQueries::moveMessageFilter(m_reader->messageFilters(), filter, false, false, filter->sortOrder() + 1, db);
+  });
+
+  m_ui.m_listFilters->insertItem(row + 1, m_ui.m_listFilters->takeItem(row));
+  m_ui.m_listFilters->setCurrentRow(row + 1);
+}
+
+void FormMessageFiltersManager::moveFilterUp() {
+  auto* filter = selectedFilter();
+  auto row = m_ui.m_listFilters->currentRow();
+
+  qApp->database()->worker()->write([&](const QSqlDatabase& db) {
+    DatabaseQueries::moveMessageFilter(m_reader->messageFilters(), filter, false, false, filter->sortOrder() - 1, db);
+  });
+
+  m_ui.m_listFilters->insertItem(row - 1, m_ui.m_listFilters->takeItem(row));
+  m_ui.m_listFilters->setCurrentRow(row - 1);
+}
+
+void FormMessageFiltersManager::openDocs() {
+  qApp->web()->openUrlInExternalBrowser(QSL("https://rssguard.readthedocs.io/en/stable/features/filters.html"));
+}
+
+void FormMessageFiltersManager::displaySelectedMessageDetails(const QModelIndex& current, const QModelIndex& previous) {
+  Q_UNUSED(previous)
+
+  QModelIndex idx = m_msgProxyModel->mapToSource(current);
+
+  if (!idx.isValid()) {
+    // NOTE: No need to clear, widget is hidden.
+    // Nothing selected, just clear.
+    /*
+    m_ui.m_tbMessageUrl->clear();
+    m_ui.m_tbMessageContents->clear();
+    m_ui.m_tbMessageDbId->clear();
+    m_ui.m_tbMessageCustomId->clear();
+    m_ui.m_tbMessageAuthor->clear();
+    */
+  }
+  else {
+    const Message* msg = m_msgModel->messageForRow(idx.row());
+
+    m_ui.m_tbMessageUrl->setText(msg->m_url);
+    m_ui.m_tbMessageContents->setPlainText(msg->m_contents);
+    m_ui.m_tbMessageDbId->setText(QString::number(msg->m_id));
+    m_ui.m_tbMessageCustomId->setText(msg->m_customId);
+    m_ui.m_tbMessageAuthor->setText(msg->m_author);
+  }
+
+  m_ui.m_twDetails->setVisible(idx.isValid());
+}
+
 ServiceRoot* FormMessageFiltersManager::selectedAccount() const {
   auto dat = m_ui.m_cmbAccounts->currentData(Qt::ItemDataRole::UserRole);
 
@@ -174,10 +309,10 @@ void FormMessageFiltersManager::filterMessagesLikeThis(const Message& msg) {
         "    msg.url == '%4';\n"
         "\n"
         "  if (is_message_same) {\n"
-        "    return MessageObject.Accept;\n"
+        "    return Msg.Accept;\n"
         "  }\n"
         "  else {\n"
-        "    return MessageObject.Ignore;\n"
+        "    return Msg.Ignore;\n"
         "  }\n"
         "}")
       .arg(QString::number(int(msg.m_isRead)), QString::number(int(msg.m_isImportant)), msg.m_title, msg.m_url);
@@ -221,23 +356,29 @@ void FormMessageFiltersManager::removeSelectedFilter() {
 void FormMessageFiltersManager::loadFilters() {
   auto flt = m_reader->messageFilters();
 
-  for (auto* fltr : std::as_const(flt)) {
+  auto flt_ordered = qlinq::from(flt).orderBy([](MessageFilter* f) {
+    return f->sortOrder();
+  });
+
+  m_ui.m_listFilters->clear();
+
+  for (auto* fltr : std::as_const(flt_ordered)) {
     auto* it = new QListWidgetItem(fltr->name(), m_ui.m_listFilters);
 
     it->setData(Qt::ItemDataRole::UserRole, QVariant::fromValue<MessageFilter*>(fltr));
+    updateItemFromFilter(it, fltr);
   }
 }
 
 void FormMessageFiltersManager::addNewFilter(const QString& filter_script) {
   try {
-    auto* fltr = m_reader->addMessageFilter(tr("New article filter"),
-                                            filter_script.isEmpty()
-                                              ? QSL("function filterMessage() { return MessageObject.Accept; }")
-                                              : filter_script);
+    auto* fltr =
+      m_reader->addMessageFilter(tr("New article filter"),
+                                 filter_script.isEmpty() ? QSL("function filterMessage() { return Msg.Accept; }")
+                                                         : filter_script);
     auto* it = new QListWidgetItem(fltr->name(), m_ui.m_listFilters);
 
     it->setData(Qt::ItemDataRole::UserRole, QVariant::fromValue<MessageFilter*>(fltr));
-
     m_ui.m_listFilters->setCurrentRow(m_ui.m_listFilters->count() - 1);
   }
   catch (const ApplicationException& ex) {
@@ -259,241 +400,109 @@ void FormMessageFiltersManager::saveSelectedFilter() {
     return;
   }
 
+  fltr->setEnabled(m_ui.m_btnEnable->isChecked());
   fltr->setName(m_ui.m_txtTitle->text());
   fltr->setScript(m_ui.m_txtScript->toPlainText());
-  m_ui.m_listFilters->currentItem()->setText(fltr->name());
+
+  updateItemFromFilter(m_ui.m_listFilters->currentItem(), fltr);
 
   m_reader->updateMessageFilter(fltr);
 }
 
+void FormMessageFiltersManager::updateItemFromFilter(QListWidgetItem* item, MessageFilter* filter) {
+  item->setText(filter->name());
+  item->setForeground(filter->enabled() ? QBrush() : QBrush(Qt::GlobalColor::red));
+}
+
 void FormMessageFiltersManager::loadFilter() {
   auto* filter = selectedFilter();
   auto* acc = selectedAccount();
 
-  loadAccount(acc);
+  updateFilterOptions(filter);
+  // loadAccount(acc);
   showFilter(filter);
   loadFilterFeedAssignments(filter, acc);
 }
 
 void FormMessageFiltersManager::testFilter() {
   m_ui.m_txtErrors->clear();
+  m_ui.m_txtErrors->setTextColor(m_defaultTextColor);
 
   // Perform per-message filtering.
   auto* selected_fd_cat = selectedCategoryFeed();
-  QJSEngine filter_engine;
-  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
-  MessageObject msg_obj(&database,
-                        selected_fd_cat->kind() == RootItem::Kind::Feed ? selected_fd_cat->toFeed() : nullptr,
-                        selectedAccount(),
-                        false);
-  auto* fltr = selectedFilter();
+  FilteringSystem filtering(FilteringSystem::FiteringUseCase::ExistingArticles,
+                            selected_fd_cat->kind() == RootItem::Kind::Feed ? selected_fd_cat->toFeed() : nullptr,
+                            selectedAccount());
 
-  MessageFilter::initializeFilteringEngine(filter_engine, &msg_obj);
+  filtering.filterRun().setTotalCountOfFilters(1);
+  filtering.filterRun().setIndexOfCurrentFilter(0);
+
+  connect(&filtering.filterApp(), &FilterApp::logged, this, [this](const QString& message) {
+    m_ui.m_txtErrors->append(message);
+  });
+
+  auto* fltr = selectedFilter();
 
   // Test real messages.
   try {
-    m_msgModel->testFilter(fltr, &filter_engine, &msg_obj);
+    m_msgModel->testFilter(fltr, &filtering);
   }
   catch (const FilteringException& ex) {
     m_ui.m_txtErrors->setTextColor(Qt::GlobalColor::red);
-    m_ui.m_txtErrors->insertPlainText(tr("EXISTING articles filtering error: '%1'.\n").arg(ex.message()));
+    m_ui.m_txtErrors->append(ex.message());
 
     // See output.
-    m_ui.m_twMessages->setCurrentIndex(2);
-  }
-
-  // Test sample message.
-  Message msg = testingMessage();
-
-  msg_obj.setMessage(&msg);
-
-  try {
-    MessageObject::FilteringAction decision = fltr->filterMessage(&filter_engine);
-
-    m_ui.m_txtErrors->setTextColor(decision == MessageObject::FilteringAction::Accept ? Qt::GlobalColor::darkGreen
-                                                                                      : Qt::GlobalColor::red);
-
-    QString answer = tr("Article will be %1.\n\n")
-                       .arg(decision == MessageObject::FilteringAction::Accept ? tr("ACCEPTED") : tr("REJECTED"));
-
-    answer += tr("Output (modified) article is:\n"
-                 "  Title = '%1'\n"
-                 "  URL = '%2'\n"
-                 "  Author = '%3'\n"
-                 "  Is read/important = '%4/%5'\n"
-                 "  Created on = '%6'\n"
-                 "  Contents = '%7'\n"
-                 "  RAW contents = '%8'")
-                .arg(msg.m_title,
-                     msg.m_url,
-                     msg.m_author,
-                     msg.m_isRead ? tr("yes") : tr("no"),
-                     msg.m_isImportant ? tr("yes") : tr("no"),
-                     QString::number(msg.m_created.toMSecsSinceEpoch()),
-                     msg.m_contents,
-                     msg.m_rawContents);
-
-    m_ui.m_txtErrors->insertPlainText(answer);
-  }
-  catch (const FilteringException& ex) {
-    m_ui.m_txtErrors->setTextColor(Qt::GlobalColor::red);
-    m_ui.m_txtErrors->insertPlainText(tr("SAMPLE article filtering error: '%1'.\n").arg(ex.message()));
-
-    // See output.
-    m_ui.m_twMessages->setCurrentIndex(2);
-  }
-}
-
-void FormMessageFiltersManager::displayMessagesOfFeed() {
-  auto* item = selectedCategoryFeed();
-
-  if (item != nullptr) {
-    m_msgModel->setMessages(item->undeletedMessages());
-  }
-  else {
-    m_msgModel->setMessages({});
+    m_ui.m_twMessages->setCurrentIndex(1);
   }
 }
 
 void FormMessageFiltersManager::processCheckedFeeds() {
   QList<RootItem*> checked = m_feedsModel->sourceModel()->checkedItems();
-  auto* fltr = selectedFilter();
-  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
-  for (RootItem* it : checked) {
-    if (it->kind() == RootItem::Kind::Feed) {
-      QJSEngine filter_engine;
-      MessageObject msg_obj(&database, it->toFeed(), selectedAccount(), false);
-
-      MessageFilter::initializeFilteringEngine(filter_engine, &msg_obj);
-
-      // We process messages of the feed.
-      QList<Message> msgs = it->undeletedMessages();
-      QList<Message> read_msgs, important_msgs;
-
-      for (int i = 0; i < msgs.size(); i++) {
-        auto labels_in_message = DatabaseQueries::getLabelsForMessage(database, msgs[i], msg_obj.availableLabels());
-
-        // Create backup of message.
-        Message* msg = &msgs[i];
-
-        msg->m_assignedLabels = labels_in_message;
-        msg->m_rawContents = Message::generateRawAtomContents(*msg);
-
-        Message msg_backup(*msg);
-
-        msg_obj.setMessage(msg);
-
-        bool remove_from_list = false;
-
-        try {
-          MessageObject::FilteringAction result = fltr->filterMessage(&filter_engine);
-
-          if (result == MessageObject::FilteringAction::Purge) {
-            remove_from_list = true;
-
-            // Purge the message completely and remove leftovers.
-            DatabaseQueries::purgeMessage(database, msg->m_id);
-          }
-          else if (result == MessageObject::FilteringAction::Ignore) {
-            remove_from_list = true;
-          }
-        }
-        catch (const FilteringException& ex) {
-          qCriticalNN << LOGSEC_CORE << "Error when running script when processing existing messages:"
-                      << QUOTE_W_SPACE_DOT(ex.message());
-
-          continue;
-        }
-
-        if (!msg_backup.m_isRead && msg->m_isRead) {
-          qDebugNN << LOGSEC_FEEDDOWNLOADER << "Message with custom ID: '" << msg_backup.m_customId
-                   << "' was marked as read by message scripts.";
-
-          read_msgs << *msg;
-        }
-
-        if (!msg_backup.m_isImportant && msg->m_isImportant) {
-          qDebugNN << LOGSEC_FEEDDOWNLOADER << "Message with custom ID: '" << msg_backup.m_customId
-                   << "' was marked as important by message scripts.";
-
-          important_msgs << *msg;
-        }
-
-        // Process changed labels.
-        for (Label* lbl : std::as_const(msg_backup.m_assignedLabels)) {
-          if (!msg->m_assignedLabels.contains(lbl)) {
-            // Label is not there anymore, it was deassigned.
-            lbl->deassignFromMessage(*msg);
-
-            qDebugNN << LOGSEC_FEEDDOWNLOADER << "It was detected that label" << QUOTE_W_SPACE(lbl->customId())
-                     << "was DEASSIGNED from message" << QUOTE_W_SPACE(msg->m_customId) << "by message filter(s).";
-          }
-        }
-
-        for (Label* lbl : std::as_const(msg->m_assignedLabels)) {
-          if (!msg_backup.m_assignedLabels.contains(lbl)) {
-            // Label is in new message, but is not in old message, it
-            // was newly assigned.
-            lbl->assignToMessage(*msg);
-
-            qDebugNN << LOGSEC_FEEDDOWNLOADER << "It was detected that label" << QUOTE_W_SPACE(lbl->customId())
-                     << "was ASSIGNED to message" << QUOTE_W_SPACE(msg->m_customId) << "by message filter(s).";
-          }
-        }
-
-        if (remove_from_list) {
-          // Do not update message.
-          msgs.removeAt(i--);
-        }
-      }
-
-      if (!read_msgs.isEmpty()) {
-        // Now we push new read states to the service.
-        if (it->getParentServiceRoot()->onBeforeSetMessagesRead(it, read_msgs, RootItem::ReadStatus::Read)) {
-          qDebugNN << LOGSEC_FEEDDOWNLOADER << "Notified services about messages marked as read by message filters.";
-        }
-        else {
-          qCriticalNN << LOGSEC_FEEDDOWNLOADER
-                      << "Notification of services about messages marked as read by message filters FAILED.";
-        }
-      }
-
-      if (!important_msgs.isEmpty()) {
-        // Now we push new read states to the service.
-        auto list = boolinq::from(important_msgs)
-                      .select([](const Message& msg) {
-                        return ImportanceChange(msg, RootItem::Importance::Important);
-                      })
-                      .toStdList();
-        QList<ImportanceChange> chngs = FROM_STD_LIST(QList<ImportanceChange>, list);
-
-        if (it->getParentServiceRoot()->onBeforeSwitchMessageImportance(it, chngs)) {
-          qDebugNN << LOGSEC_FEEDDOWNLOADER
-                   << "Notified services about messages marked as important by message filters.";
-        }
-        else {
-          qCriticalNN << LOGSEC_FEEDDOWNLOADER
-                      << "Notification of services about messages marked as important by message filters FAILED.";
-        }
-      }
-
-      // Update messages in DB and reload selection.
-      it->getParentServiceRoot()->updateMessages(msgs, it->toFeed(), true, nullptr);
-      displayMessagesOfFeed();
-    }
+  try {
+    m_msgModel->processFeeds(selectedFilter(), selectedAccount(), checked);
   }
+  catch (const ApplicationException& ex) {
+    m_ui.m_txtErrors->setTextColor(Qt::GlobalColor::red);
+    m_ui.m_txtErrors->append(ex.message());
+
+    // See output.
+    m_ui.m_twMessages->setCurrentIndex(1);
+  }
+
+  onFeedChanged();
+}
+
+void FormMessageFiltersManager::onFeedChanged() {
+  auto* item = selectedCategoryFeed();
+
+  if (item != nullptr) {
+    MessagesModel loader;
+    loader.loadMessages(item);
+
+    m_msgModel->setMessages(loader.messages());
+  }
+  else {
+    m_msgModel->setMessages({});
+  }
+
+  displaySelectedMessageDetails(m_ui.m_treeExistingMessages->currentIndex(), {});
 }
 
 void FormMessageFiltersManager::loadAccount(ServiceRoot* account) {
   m_feedsModel->setRootItem(account, false, true);
 
   if (account != nullptr) {
-    m_msgModel->setMessages(account->undeletedMessages());
+    MessagesModel loader;
+    loader.loadMessages(account);
+
+    m_msgModel->setMessages(loader.messages());
   }
   else {
     m_msgModel->setMessages({});
   }
+
+  displaySelectedMessageDetails(m_ui.m_treeExistingMessages->currentIndex(), {});
 }
 
 void FormMessageFiltersManager::loadFilterFeedAssignments(MessageFilter* filter, ServiceRoot* account) {
@@ -502,12 +511,13 @@ void FormMessageFiltersManager::loadFilterFeedAssignments(MessageFilter* filter,
   }
 
   m_loadingFilter = true;
+
   auto stf = account->getSubTreeFeeds();
 
   for (auto* feed : std::as_const(stf)) {
-    if (feed->messageFilters().contains(filter)) {
-      m_feedsModel->sourceModel()->setItemChecked(feed, Qt::CheckState::Checked);
-    }
+    m_feedsModel->sourceModel()->setItemChecked(feed,
+                                                feed->messageFilters().contains(filter) ? Qt::CheckState::Checked
+                                                                                        : Qt::CheckState::Unchecked);
   }
 
   m_loadingFilter = false;
@@ -573,7 +583,6 @@ void FormMessageFiltersManager::showFilter(MessageFilter* filter) {
   }
 
   // See message.
-  m_ui.m_twMessages->setCurrentIndex(0);
   m_loadingFilter = false;
 }
 
@@ -619,7 +628,7 @@ void FormMessageFiltersManager::beautifyScript() {
                    QMessageBox::Icon::Critical,
                    tr("Error"),
                    tr("Script was not beautified, because 'clang-format' tool thrown error."),
-                   QString(),
+                   {},
                    err);
     }
   }
@@ -632,58 +641,14 @@ void FormMessageFiltersManager::beautifyScript() {
   }
 }
 
-void FormMessageFiltersManager::insertPremadeFilter(QAction* act_filter) {
-  QString filter_path = QSL(":/scripts/filters/") + act_filter->text();
-
-  try {
-    m_ui.m_txtScript->setPlainText(QString::fromUtf8(IOFactory::readFile(filter_path)));
-  }
-  catch (...) {
-  }
-}
-
-void FormMessageFiltersManager::initializePremadeFilters() {
-  QMenu* mn_filters = new QMenu(this);
-
-  connect(mn_filters, &QMenu::triggered, this, &FormMessageFiltersManager::insertPremadeFilter);
-
-  auto js_files = QDir(QSL(":/scripts/filters")).entryList();
-
-  for (const QString& js_file : js_files) {
-    mn_filters->addAction(js_file);
-  }
-
-  m_ui.m_btnPremadeFilters->setMenu(mn_filters);
-}
-
-void FormMessageFiltersManager::initializeTestingMessage() {
-  m_ui.m_cbSampleImportant->setChecked(true);
-  m_ui.m_txtSampleUrl->setText(QSL("https://mynews.com/news/5"));
-  m_ui.m_txtSampleTitle->setText(QSL("Year of Linux Desktop"));
-  m_ui.m_txtSampleAuthor->setText(QSL("Napoleon Bonaparte"));
-  m_ui.m_txtSampleContents->setPlainText(QSL("<p>Browsers usually insert quotation marks around the q element.</p>"
-                                             "<p>WWF's goal is to: <q>Build a future where people live in harmony "
-                                             "with nature.</q></p>"));
-  m_ui.m_txtSampleCreatedOn->setText(QString::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch()));
+void FormMessageFiltersManager::updateFilterOptions(MessageFilter* filter) {
+  m_ui.m_btnRemoveSelected->setEnabled(filter != nullptr);
+  m_ui.m_btnUp->setEnabled(filter != nullptr && filter->sortOrder() > 0);
+  m_ui.m_btnDown->setEnabled(filter != nullptr && filter->sortOrder() < m_reader->messageFilters().size() - 1);
+  m_ui.m_btnEnable->setEnabled(filter != nullptr);
+  m_ui.m_btnEnable->setChecked(filter != nullptr && filter->enabled());
 }
 
 RootItem* FormMessageFiltersManager::selectedCategoryFeed() const {
   return m_feedsModel->sourceModel()->itemForIndex(m_feedsModel->mapToSource(m_ui.m_treeFeeds->currentIndex()));
 }
-
-Message FormMessageFiltersManager::testingMessage() const {
-  Message msg;
-
-  msg.m_feedId = QString::number(NO_PARENT_CATEGORY);
-  msg.m_url = m_ui.m_txtSampleUrl->text();
-  msg.m_customId = m_ui.m_txtSampleUrl->text();
-  msg.m_title = m_ui.m_txtSampleTitle->text();
-  msg.m_author = m_ui.m_txtSampleAuthor->text();
-  msg.m_isRead = m_ui.m_cbSampleRead->isChecked();
-  msg.m_isImportant = m_ui.m_cbSampleImportant->isChecked();
-  msg.m_created = QDateTime::fromMSecsSinceEpoch(m_ui.m_txtSampleCreatedOn->text().toLongLong());
-  msg.m_contents = m_ui.m_txtSampleContents->toPlainText();
-  msg.m_rawContents = Message::generateRawAtomContents(msg);
-
-  return msg;
-}
